=== modified file 'bzrlib/tests/per_tree/test_tree.py'
--- bzrlib/tests/per_tree/test_tree.py	2012-01-30 14:18:22 +0000
+++ bzrlib/tests/per_tree/test_tree.py	2012-02-21 13:35:17 +0000
@@ -249,6 +249,29 @@
                          output_order)
 
 
+class TestIterChildEntries(TestCaseWithTree):
+
+    def test_iteration_order(self):
+        work_tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b/', 'a/b/c', 'a/d/', 'a/d/e', 'f/', 'f/g'])
+        work_tree.add(['a', 'a/b', 'a/b/c', 'a/d', 'a/d/e', 'f', 'f/g'])
+        tree = self._convert_tree(work_tree)
+        output = [e.name for e in
+            tree.iter_child_entries(tree.get_root_id())]
+        self.assertEqual(set(['a', 'f']), set(output))
+        output = [e.name for e in
+            tree.iter_child_entries(tree.path2id('a'))]
+        self.assertEqual(set(['b', 'd']), set(output))
+
+    def test_does_not_exist(self):
+        work_tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/'])
+        work_tree.add(['a'])
+        tree = self._convert_tree(work_tree)
+        self.assertRaises(errors.NoSuchId, lambda:
+            list(tree.iter_child_entries('unknown')))
+
+
 class TestHasId(TestCaseWithTree):
 
     def test_has_id(self):

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2012-02-06 23:38:33 +0000
+++ bzrlib/transform.py	2012-02-21 13:35:17 +0000
@@ -2183,6 +2183,14 @@
                 ordered_ids.append((trans_id, parent_file_id))
         return ordered_ids
 
+    def iter_child_entries(self, file_id, path=None):
+        self.id2path(file_id)
+        trans_id = self._transform.trans_id_file_id(file_id)
+        todo = [(child_trans_id, trans_id) for child_trans_id in
+                self._all_children(trans_id)]
+        for entry, trans_id in self._make_inv_entries(todo):
+            yield entry
+
     def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
         # This may not be a maximally efficient implementation, but it is
         # reasonably straightforward.  An implementation that grafts the

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2012-02-21 12:53:02 +0000
+++ bzrlib/tree.py	2012-02-21 13:35:17 +0000
@@ -200,7 +200,8 @@
 
         :param file_id: File id of the directory/tree-reference
         :param path: Optional path of the directory
-        :return: Iterator over (name, entry) tuples
+        :raise NoSuchId: When the file_id does not exist
+        :return: Iterator over entries in the directory
         """
         raise NotImplementedError(self.iter_child_entries)
 
@@ -883,7 +884,7 @@
     @needs_read_lock
     def iter_child_entries(self, file_id, path=None):
         inv, inv_file_id = self._unpack_file_id(file_id)
-        return inv[inv_file_id].children.iteritems()
+        return inv[inv_file_id].children.itervalues()
 
     @deprecated_method(deprecated_in((2, 5, 0)))
     def get_file_by_path(self, path):

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2012-02-18 16:55:04 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2012-02-21 13:35:17 +0000
@@ -51,6 +51,8 @@
 .. Changes that may require updates in plugins or other code that uses
    bzrlib.
 
+* New API method ``Tree.iter_child_entries``.  (Jelmer Vernooij)
+
 Internals
 *********
 

